
/* Erstellt: 2023-03-13
 * Version V1.0
 * Autor:  Roland Kaiser, OE2ROL
 * Geschrieben für Arduino Nano ATmega 328P(Old Bootloader)
 * Der Autor übernimmt keine Haftung etwaiger Folgeschäden an Transceivern oder dgl.
 * 
 *   
 * Diese Software ist für den eigenen privaten Gebrauch frei verfügbar, jedoch nicht für kommerzielle Zwecke ohne Genehmigung des Autors.
 * Veränderungen und Verbesserungen der Software bitte wieder an den Autor Roland Kaiser OE2ROL (roland.kaiser@sbg.at) senden.
 * Die Software wurde am ICOM IC-9700 getestet.
 * 
 * Funktionsweise:
 *   
 * Automatische Einstellung der Sendeleistung im SAT-Mode
 * Steuerung eines Koaxrelais (12V Schaltausgang) zum automatischen Umschalten der Antennenbuchsen zwischen terrestrischen Antennen und im SAT-Mode auf Up- bzw Downconverter 
 * 
 * Änderungsübersicht:
 * V1.0 initiale Version
 *      
 * Anschlussbelegung der Ports:
   
   CAT port GND -> Arduino GND
   CAT port TX,RX  -> Arduino pin 11,12
 */


#include <SoftwareSerial.h>

byte civadr = 0xA2;                   // CI-V Adresse
const int AntSWPin = 13;              // Pin für Ant SW 

SoftwareSerial CAT(11, 12);           // RX, TX Pin CAT Schnittstelle


void setup(){  
CAT.begin(9600);                      // CAT schnittstelle initialisieren
Serial.begin(9600);                   // Serial Monitor Init
delay(100);                           // Verzögerung
pinMode(AntSWPin, OUTPUT);            // Setzt den Analogpin A6 als Outputpin
}



// Sendeleistung am Transceiver setzen

void set_TXPower() {
uint8_t power[] = {0xFE, 0xFE, civadr, 0x00, 0x14, 0x0A, 0x00, 0x00, 0xFD};   // CAT Kommando zum setzen der TX Power (7.Byte 0x00 0x00 = Minimum bis 8.Byte 0x02 0x55 = Maximum)
for (uint8_t i = 0; i < sizeof(power); i++) {                                 // Zählschleife 0 - Grösse des Schreibarrays
  CAT.write(power[i]);                                                        // CAT Kommando senden
  }
delay(20);                                                                    // Verzögerung 20 ms  
}



// MAIN Frequenz am Transceiver auswählen

void select_main() {
uint8_t main[] = {0xFE, 0xFE, civadr, 0x00, 0x07, 0xD0, 0xFD};          // CAT Kommando zum Umschalten auf die Main Frequenz
for (uint8_t i = 0; i < sizeof(main); i++) {                            // Zählschleife 0 - Grösse des Schreibarrays
  CAT.write(main[i]);                                                   // CAT Kommando senden
  }
delay(20);                                                              // Verzögerung 20 ms
}



// SUB Frequenz am Transceiver auswählen

void select_sub() {
uint8_t sub[] = {0xFE, 0xFE, civadr, 0x00, 0x07, 0xD1, 0xFD};           // CAT Kommando zum Umschalten auf die Sub Frequenz
for (uint8_t i = 0; i < sizeof(sub); i++) {                             // Zählschleife 0 - Grösse des Schreibarray
  CAT.write(sub[i]);                                                    // CAT Kommando senden
  }
delay(20);                                                              // Verzögerung 20 ms
}



// SAT MODE vom Transceiver auslesen und entsprechende Funktionen ausführen

void read_satmode() {
uint8_t satmode_buffer[12];                                             // Array zum Lesen des Sat-Modes
uint8_t satmode[] = {0xFE, 0xFE, civadr, 0x00, 0x16, 0x5A, 0xFD};       // CAT Kommando zum auslesen des Satmodes  
for (uint8_t i = 0; i < sizeof(satmode); i++) {                         // Grösse des Schreibarrays 
  CAT.write(satmode[i]);                                                // CAT Kommando senden
  }
delay(20);                                                              // Verzögerung 20 ms
  while (!CAT.available());                                             // auf Serial Port warten
    while (CAT.available() > 0) {                                       // wenn Serial Port verfügbar
      for (int j = 0; j < 12; j++) {                                    // Zählschleife 0-12
        satmode_buffer[j] = CAT.read();                                 // Zeichen vom Serial Port schrittweise in das Array schreiben
          if (satmode_buffer[j] == 0xFE){                               // wenn 0xFE im Buffer steht
            j = 0;                                                      // Arrayindexzähler auf 0 setzen
          }
      }                                                                 // FE FE 00 A2 <01> FD = Sat Mode ein, <00> Sat Mode aus
    }
    if (satmode_buffer[5] == 0x01) {                                    // wenn Sat-Mode ein ->
      set_TXPower();                                                    // Setzen der Sendeleistung ausführen
      digitalWrite(AntSWPin, HIGH);                                     // Relaisausgang für Antennenschalter auf High setzen
    }
    else if (satmode_buffer[5] == 0x00) {                               // wenn Sat-Mode aus ->
     digitalWrite(AntSWPin, LOW);                                       // Relaisausgang für Antennenschalter auf Low setzen
    } 
} 



void loop(){

read_satmode();                                         // Sat Mode lesen ausführen
delay(1000);                                            // Verzögerung 1000ms
}
